<?php

namespace Drupal\find_text\Form;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\taxonomy\Entity\Term;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Cache\CacheTagsInvalidatorInterface;

/**
 * Configure Search settings for this site.
 */
class SearchForm extends ConfigFormBase {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The EntityTypeManager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * For Managing cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheManager;

  /**
   * Cache Tags Invalidator.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  protected $cacheTagsInvalidator;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->renderer = $container->get('renderer');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->cacheManager = $container->get('cache.data');
    $instance->cacheTagsInvalidator = $container->get('cache_tags.invalidator');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'find_text';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $raw_query_string = \Drupal::request();
    $raw_query_search_text = $raw_query_string->get('text');
    $raw_query_render_as_markup = $raw_query_string->get('markup');
    $raw_query_regex = $raw_query_string->get('regex');

    $query_search_text = $raw_query_search_text ? urldecode($raw_query_search_text): NULL;
    $query_render_as_markup = $raw_query_render_as_markup ? $raw_query_render_as_markup === 'true' : NULL;
    $query_regex = $raw_query_regex ? $raw_query_regex === 'true' : NULL;

    $wrapper_id = $this->getFormId() . '-wrapper';

    $form = [
      '#prefix' => '<div id="' . $wrapper_id . '" aria-live="polite">',
      '#suffix' => '</div>',
    ];
    $form['intro'] = [
      '#type' => 'markup',
      '#markup' => <<< 'EOD'
        <p>Search text fields for a provided string. The search is not case-sensitive.</p>
        <p>Pre-rendered text area markup is searched, so some characters may be different; for instance, `&amp ;` may match ampersands where `&` will not.
        Node fields and content blocks are included, but some areas such as menu links will not be searched.</p>
      EOD,
    ];
    $form['needle'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search Text'),
      '#default_value' => $query_search_text ?: '',
      '#description' => $this->t('The string to search against. Wildcards % will match any number of characters and _ will match any single character. % wildcards are prepended and appended automatically when not using regex. See the <a href="https://sitenow.uiowa.edu/documentation/site-text-search">Find Text documentation</a> for more information.'),
    ];
    $form['render'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Render markup'),
      '#description' => $this->t('Display the results as rendered HTML. This may hide parts of the results, such as text matched within HTML tags.'),
      '#default_value' => $query_render_as_markup !== NULL ? $query_render_as_markup : 1,
    ];
    $form['regexed'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('REGEXP?'),
      '#description' => $this->t('% wildcards will not be prepended or appended, and the search will be performed as a REGEXP search rather than LIKE. Use only if a full regular expression is required.'),
      '#default_value' =>  $query_regex !== NULL ? $query_regex : 0,
    ];
    // Show invalidate cache checkbox only if search results caching is enabled.
    if ($this->configFactory()->get('find_text.settings')->get('enable_search_results_cache')) {
      $form['invalidate_cache'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Invalidate cache'),
        '#description' => $this->t('If checked, the cache for search results will be cleared and the search will be performed again.'),
        '#default_value' => 0,
      ];
    }
    $form['search'] = [
      '#type' => 'submit',
      '#value' => $this->t('Search'),
      '#button_type' => 'primary',
      '#name' => 'search',
      '#submit' => [
        [$this, 'searchButton'],
      ],
      '#ajax' => [
        'callback' => [$this, 'searchButton'],
        'wrapper' => $wrapper_id,
        'method' => 'html',
        'disable-refocus' => TRUE,
        'effect' => 'fade',
        'event' => 'click',
      ],
    ];
    $form['actions']['reset'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Reset'),
      '#name' => 'reset',
      '#submit' => [
        [$this, 'resetButton'],
      ],
    );

    // Unset the original, currently unused submit button.
    // It might be used at another time if settings are needed.
    unset($form['actions']['submit']);
    // Attach our custom css for highlighting purposes.
    $form['#attached']['library'][] = 'find_text/results';
    return $form;
  }

  /**
   * Perform a search with the given needle.
   */
  public function searchButton(array &$form, FormStateInterface $form_state) {
    $needle = $form_state->getValue('needle');
    // Only handle the processing if we have
    // a needle length of 3 or more characters.
    if (strlen($needle) >= 3) {
      $regexed = $form_state->getValue('regexed');
      $render = $form_state->getValue('render');
      // Clear the cache for search results if invalidate cache option
      // is selected.
      if ($form_state->getValue('invalidate_cache')) {
        $this->cacheTagsInvalidator->invalidateTags(['search_results:data']);
      }
      $find_text_settings = $this->configFactory()->get('find_text.settings');
      $is_cache_enabled = $find_text_settings->get('enable_search_results_cache');
      $cid = 'search_results:' . md5($needle);
      // Check if search results caching is enabled and data is
      // available in the cache.
      if ($is_cache_enabled && $cache = $this->cacheManager->get($cid)) {
        $results = $cache->data['results'];
        $table = $cache->data['table'];
      }
      else {
        $results = search_fields($needle, $regexed, $render);
        $table = $this->buildResultsTable($results);
        // Store the data in cache if search results caching is enabled.
        if ($is_cache_enabled) {
          $cache_expiry = time() + $find_text_settings->get('search_results_cache_duration') ?? 3600;
          // Need to store results and table both as we need results array
          // to generate the search results CSV.
          $this->cacheManager->set(
            'search_results:' . md5($needle),
            [
              'results' => $results,
              'table' => $table,
            ],
            $cache_expiry,
            ['search_results:data']
          );
        }
      }
      $markup = $this->renderer->render($table);
      $form['results'] = [
        '#type' => 'markup',
        '#markup' => $markup,
      ];
      if ($find_text_settings->get('save_as_csv')) {
        $response = $this->generateCsvFile($results);
        $form_state->setResponse($response);
      }
    }
    return $form;
  }

  /**
   * Helper function to generate a CSV file from search results.
   *
   * @param array $results
   *   The search form results to export.
   *
   * @return \Symfony\Component\HttpFoundation\BinaryFileResponse
   *   The binary response containing the file.
   */
  public function generateCsvFile(array $results) {
    $file_directory = $this->configFactory()->get('system.file')->get('default_scheme') . '://';
    $file_name = 'search_results_' . date('Y-m-d_H-i-s') . '.csv';
    $csv_file_path = $file_directory . $file_name;
    $csv_file = fopen($csv_file_path, 'w');
    $header = ['Entity', 'Field: Contents'];
    fputcsv($csv_file, $header);
    foreach ($results['node'] as $node_data) {
      foreach ($node_data as $node_item) {
        $row = [
          $node_item->entity_id,
          $node_item->body_value,
        ];
        fputcsv($csv_file, $row);
      }
    }
    fclose($csv_file);
    $response = new BinaryFileResponse($csv_file_path);
    $response->headers->set('Content-Type', 'text/csv');
    $response->setContentDisposition(
      ResponseHeaderBag::DISPOSITION_ATTACHMENT,
      $file_name
    );
    return $response;
  }

  /**
   * Resets the form back to the default state
   */
  function resetButton(array &$form, FormStateInterface $form_state) {
    $form_state->setRebuild(false);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getTriggeringElement()['#name'] === 'search' && strlen($form_state->getValue('needle')) < 3) {
      $form_state->setErrorByName('needle', $this->t('The search target must contain 3 or more characters.'));
    }
  }

  /**
   * Helper function to form our table array.
   *
   * @param array $results
   *   The search form results to prepare.
   *
   * @return array
   *   The renderable table array.
   */
  public function buildResultsTable(array $results) {
    if (empty($results)) {
      return [
        '#type' => 'markup',
        '#markup' => '<p class="text-align-center">No results found.</p>',
      ];
    }
    // Rearrange and clear out the excess to make printing easier.
    // Starting out, our results are separated by entity type.
    foreach ($results as $type => $typed_results) {
      // This first key will be the entity id.
      foreach (array_keys($typed_results) as $key) {
        // The secondary key will be a simple delta.
        foreach (array_keys($typed_results[$key]) as $secondary_key) {
          // Created a modified key to help us avoid collisions
          // while rearranging our results.
          $mod_key = implode('-', [$type, $key]);
          $results[$mod_key][] = $results[$type][$key][$secondary_key]->value;
        }
      }
      unset($results[$type]);
    }
    $rows = [];
    $node_manager = $this->entityTypeManager
      ->getStorage('node');
    foreach ($results as $mod_key => $matches) {
      $exploded = explode('-', $mod_key);
      [$type, $id] = $exploded;
      switch ($type) {
        case 'block_content':
        case 'paragraph':
        case 'node':
          $node = $node_manager->load($id);

          //Check if workflow is being used. If it is, use the current workflow status.
          // If it is not, use the publish status.
          if($node->hasField('moderation_state')) {
            if($node->get('moderation_state')->value) {
              $status = $node->get('moderation_state')->value;
            }
          }
          else {
            if ($node->isPublished()) {
              $status = 'Published';
            } else {
              $status = 'Not Published';
            }
          }
          // Check if we have an overridden layout or not.
          // If we didn't successfully load a node, go ahead
          // and treat it as a non-overridden node so that
          // the user will still see it in the results as a failsafe.
          $has_lb = $node && $node->hasField('layout_builder__layout');
          if ($has_lb) {
            $entity_value = new FormattableMarkup('<a href="/node/@nid/">@title</a><br /><strong>Node:</strong> @nid (<a href="/node/@nid/edit">edit</a>) (<a href="/node/@nid/layout">layout</a>)<br /><strong>Type:</strong> @type<br /><strong>Status:</strong> @status<br /><strong>Author:</strong> @author<br /><strong>Created Date:</strong> @date<br /><strong>Last Updated:</strong> @update', [
              '@nid' => $id,
              '@title' => $node->getTitle(),
              '@type' => node_get_type_label($node),
              '@status' => $status,
              '@author' => $node->getOwner()->getDisplayName(),
              '@date' => date("m-d-Y", $node->getCreatedTime()),
              '@update' => date("m-d-Y", $node->changed->value),

            ]);
          }
          else {
            $entity_value = new FormattableMarkup('<a href="/node/@nid/">@title</a><br /><strong>Node:</strong> @nid (<a href="/node/@nid/edit">edit</a>)<br /><strong>Type:</strong> @type<br /><strong>Status:</strong> @status<br /><strong>Author:</strong> @author<br /><strong>Created Date:</strong> @date<br /><strong>Last Updated:</strong> @update', [
              '@nid' => $id,
              '@title' => $node->getTitle(),
              '@type' => node_get_type_label($node),
              '@status' => $status,
              '@author' => $node->getOwner()->getDisplayName(),
              '@date' => date("m-d-Y", $node->getCreatedTime()),
              '@update' => date("m-d-Y", $node->changed->value),
            ]);
          }
          break;

        case 'menu_link_content':
          $menu_link_manager = $this->entityTypeManager->getStorage('menu_link_content');
          $menu_link = $menu_link_manager->load($id);
          $entity_value = new FormattableMarkup('@title<br /><strong>Menu:</strong> @mid (<a href="/admin/structure/menu/item/@mid/edit?destination=/admin/find-text">edit</a>)', [
            '@mid' => $id,
            '@title' => $menu_link->getTitle(),
          ]);
          break;

        case 'taxonomy_term':
          $entity_value = new FormattableMarkup('<a href="/taxonomy/term/@tid/">@title</a></br /><strong>Term:</strong> @tid (<a href="/taxonomy/term/@tid/edit?destination=/admin/find-text">edit</a>)', [
            '@tid' => $id,
            '@title' => Term::load($id)->get('name')->value,
          ]);
          break;

        default:
          $entity_value = FALSE;
      }
      if (!$entity_value) {
        continue;
      }
      $rows[] = [
        'id' => [
          'data' => $entity_value,
          // Stretch the node row to cover all its matches.
          'rowspan' => count($matches),
        ],
        // Shift the first element out of the ray to match
        // to the node row.
        'field' => array_shift($matches),
      ];
      // If we have more matches after the shift, we need to
      // add each to its own row.
      foreach ($matches as $match) {
        $rows[] = ['field' => $match];
      }
    }
    $table = [
      '#type' => 'table',
      '#header' => [
        'nid' => 'Entity',
        'field' => 'Field: Contents',
      ],
      '#rows' => $rows,
      '#attributes' => NULL,
    ];
    return $table;
  }

}
